{*******************************************************************}
{                                                                   }
{       Vita Voom Software                                          }
{       pgeDriverUtils.pas unit                                     }
{                                                                   }
{       Copyright (c) 1998-2003 Vita Voom Software                  }
{       ALL RIGHTS RESERVED                                         }
{*******************************************************************}
{                                                                   }
{ This unit contains utility functions for use with the pgExpress   }
{ driver.                                                           }
{                                                                   }
{ Please refer to the pgeExpress documentation for details.         }
unit pgeDriverUtils;

interface

uses
  SqlExpr;

const
  strTTY = 'TTY';
  strBackendPID = 'BackendPID';
  strRetainCursor = 'RetainCursor';
  strPGVersion = 'PGVersion';

{$IFDEF VER150}
  // Retrieves the PostgreSQL version number. D7+ Only.
  function GetPGVersion(Connection: TSQLConnection): Extended;

  // Returns the file or TTY debug for the connection. D7+ Only.
  function GetTTY(Connection: TSQLConnection): AnsiString;

  // Returns the server PID (process ID) for the connection. D7+ Only.
  function GetBackendPID(Connection: TSQLConnection): Integer;

  // Returns True if a connection to the server exists. D7+ Only.
  function GetOnline(Connection: TSQLConnection): Boolean;

  // Returns the internal RetainCursor variable. D7+ Only.
  // Obs: since this feature is experimental, there is no entry on the documentaion for it.
  function GetRetainCursor(Connection: TSQLConnection): Boolean;

  // Sets the internal RetainCursor variable. D7+ Only.
  // Obs: since this feature is experimental, there is no entry on the documentaion for it.
  procedure SetRetainCursor(Connection: TSQLConnection; Value: Boolean);
{$ENDIF}  

implementation

uses
  Types, SysUtils, DBXpress, DB;

// Auxiliar function.
procedure Check(Value: Word; Connection: TSQLConnection);
var
  S: AnsiString;
  Len: SmallInt;
begin
  if Value <> SQL_SUCCESS then
  begin
    Len := Connection.SQLConnection.getErrorMessageLen(Len);
    SetLength(S, Len);
    Connection.SQLConnection.getErrorMessage(PChar(S));
    DatabaseError(S);
  end;  
end;

{$IFDEF VER150}
  // Retrieves the PostgreSQL version number. D7+ Only.
  function GetPGVersion(Connection: TSQLconnection): Extended;
  const
    BufSize: Byte = 50;
  var
    Len: SmallInt;
    Buffer: TByteDynArray;
  begin
    SetLength(Buffer, BufSize);
    StrCopy(@Buffer[0], strPGVersion);
    Check(Connection.SQLConnection.GetOption(eConnCustomInfo, @Buffer[0], Length(Buffer), Len), Connection);
    Result := PExtended(Buffer)^;
  end;

  // Returns the file or TTY debug for the connection. D7+ Only.
  function GetTTY(Connection: TSQLConnection): AnsiString;
  const
    BufSize: Byte = 50;
  var
    Buffer: TByteDynArray;
    Len: SmallInt;
  begin
    SetLength(Buffer, BufSize);
    try
      StrCopy(@Buffer[0], strTTY);
      Check(Connection.SQLConnection.GetOption(eConnCustomInfo, @Buffer[0], BufSize, Len), Connection);
      Result := StrPas(@Buffer[0]);
    finally
      Buffer := nil;
    end;
  end;

  // Returns the file or TTY debug for the connection. D7+ Only.
  function GetBackendPID(Connection: TSQLConnection): Integer;
  const
    BufSize: Byte = 20;
  var
    Buffer: TByteDynArray;
    Len: SmallInt;
  begin
    SetLength(Buffer, BufSize);
    try
      StrCopy(@Buffer[0], strBackendPID);
      Check(Connection.SQLConnection.GetOption(eConnCustomInfo, @Buffer[0], BufSize, Len), Connection);
      Result := PInteger(@Buffer[0])^;
    finally
      Buffer := nil;
    end;
  end;

  // Returns True if a connection to the server exists. D7+ Only.
  function GetOnline(Connection: TSQLConnection): Boolean;
  var
    Len: SmallInt;
    Value: Boolean;
  begin
    Check(Connection.SQLConnection.GetOption(eConnCustomInfo, @Value, SizeOf(Boolean), Len), Connection);
    Result := PBoolean(@Value)^;
  end;

  // Returns the internal RetainCursor variable. D7+ Only.
  function GetRetainCursor(Connection: TSQLConnection): Boolean;
  const
    BufSize: Byte = 20;
  var
    Buffer: TByteDynArray;
    Len: SmallInt;
  begin
    SetLength(Buffer, BufSize);
    try
      StrCopy(@Buffer[0], strRetainCursor);
      Check(Connection.SQLConnection.GetOption(eConnCustomInfo, @Buffer[0], BufSize, Len), Connection);
      Result := PBoolean(@Buffer[0])^;
    finally
      Buffer := nil;
    end;
  end;

  // Sets the internal RetainCursor variable. D7+ Only.
  procedure SetRetainCursor(Connection: TSQLConnection; Value: Boolean);
  var
    S: AnsiString;
  begin
    S := strRetainCursor + '=' + BoolToStr(Value, True);
    Check(Connection.SQLConnection.SetOption(eConnCustomInfo, Integer(PChar(S))), Connection);
  end;
{$ENDIF}
end.
